@@ -3,7 +3,7 @@
 # A connection pool allowing multi-threaded access to a pool of connections.
 # This is the default connection pool used by Sequel.
 class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
-  USE_WAITER = true
+  USE_WAITER = true # SEQUEL6: Remove
   Sequel::Deprecation.deprecate_constant(self, :USE_WAITER)
 
   # The maximum number of connections this pool will create (per shard/server
@@ -11,17 +11,18 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   attr_reader :max_size
   
   # An array of connections that are available for use by the pool.
-  attr_reader :available_connections
+  # The calling code should already have the mutex before calling this.
+  attr_reader :available_connections # SEQUEL6: Remove
   
-  # A hash with thread keys and connection values for currently allocated
-  # connections.
-  attr_reader :allocated
+  # A hash with thread/fiber keys and connection values for currently allocated connections.
+  # The calling code should already have the mutex before calling this.
+  attr_reader :allocated # SEQUEL6: Remove
 
   # The following additional options are respected:
   # :max_connections :: The maximum number of connections the connection pool
   #                     will open (default 4)
   # :pool_timeout :: The amount of seconds to wait to acquire a connection
-  #                  before raising a PoolTimeoutError (default 5)
+  #                  before raising a PoolTimeout error (default 5)
   def initialize(db, opts = OPTS)
     super
     @max_size = Integer(opts[:max_connections] || 4)
@@ -30,6 +31,7 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
     @connection_handling = opts[:connection_handling]
     @available_connections = []
     @allocated = {}
+    @allocated.compare_by_identity
     @timeout = Float(opts[:pool_timeout] || 5)
     @waiter = ConditionVariable.new
   end
@@ -48,8 +50,7 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
     end
   end
   
-  # Removes all connections currently available, optionally
-  # yielding each connection to the given block. This method has the effect of 
+  # Removes all connections currently available. This method has the effect of 
   # disconnecting from the database, assuming that no connections are currently
   # being used.  If you want to be able to disconnect connections that are
   # currently in use, use the ShardedThreadedConnectionPool, which can do that.
@@ -82,7 +83,7 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   # is available or the timeout expires.  If the timeout expires before a
   # connection can be acquired, a Sequel::PoolTimeout is raised.
   def hold(server=nil)
-    t = Thread.current
+    t = Sequel.current
     if conn = owned_connection(t)
       return yield(conn)
     end
@@ -101,7 +102,12 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
       end
       raise
     ensure
-      sync{release(t)} if conn
+      if conn
+        sync{release(t)}
+        if @connection_handling == :disconnect
+          disconnect_connection(conn)
+        end
+      end
     end
   end
 
@@ -128,31 +134,31 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   # calling this.
   #
   # This should return a connection is one is available within the timeout,
-  # or nil if a connection could not be acquired within the timeout.
+  # or raise PoolTimeout if a connection could not be acquired within the timeout.
   def acquire(thread)
     if conn = assign_connection(thread)
       return conn
     end
 
-    time = Time.now
+    timeout = @timeout
+    timer = Sequel.start_timer
 
     sync do
-      @waiter.wait(@mutex, @timeout)
+      @waiter.wait(@mutex, timeout)
       if conn = next_available
         return(@allocated[thread] = conn)
       end
     end
 
     until conn = assign_connection(thread)
-      deadline ||= time + @timeout
-      current_time = Time.now
-      raise_pool_timeout(current_time - time) if current_time > deadline
-
+      elapsed = Sequel.elapsed_seconds_since(timer)
       # :nocov:
+      raise_pool_timeout(elapsed) if elapsed > timeout
+
       # It's difficult to get to this point, it can only happen if there is a race condition
-      # where a connection cannot be acquired even after the thread is signalled by the condition
+      # where a connection cannot be acquired even after the thread is signalled by the condition variable
       sync do
-        @waiter.wait(@mutex, deadline - current_time)
+        @waiter.wait(@mutex, timeout - elapsed)
         if conn = next_available
           return(@allocated[thread] = conn)
         end
@@ -166,16 +172,23 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   # Assign a connection to the thread, or return nil if one cannot be assigned.
   # The caller should NOT have the mutex before calling this.
   def assign_connection(thread)
+    # Thread safe as instance variable is only assigned to local variable
+    # and not operated on outside mutex.
     allocated = @allocated
-
     do_make_new = false
+    to_disconnect = nil
+
     sync do
       if conn = next_available
         return(allocated[thread] = conn)
       end
 
       if (n = _size) >= (max = @max_size)
-        allocated.keys.each{|t| release(t) unless t.alive?}
+        allocated.keys.each do |t|
+          unless t.alive?
+            (to_disconnect ||= []) << allocated.delete(t)
+          end
+        end
         n = nil
       end
 
@@ -184,6 +197,10 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
       end
     end
 
+    if to_disconnect
+      to_disconnect.each{|dconn| disconnect_connection(dconn)}
+    end
+
     # Connect to the database outside of the connection pool mutex,
     # as that can take a long time and the connection pool mutex
     # shouldn't be locked while the connection takes place.
@@ -252,9 +269,7 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   def release(thread)
     conn = @allocated.delete(thread)
 
-    if @connection_handling == :disconnect
-      disconnect_connection(conn)
-    else
+    unless @connection_handling == :disconnect
       checkin_connection(conn)
     end
 
