@@ -21,6 +21,17 @@ describe "A MSSQL database" do
     @db.server_version
     @db.dataset.server_version
   end
+
+
+  it "should support LIKE without COLLATE" do
+    begin
+      like_without_collate = @db.like_without_collate
+      @db.like_without_collate = true
+      @db.get(Sequel.case({Sequel.like('a', 'a')=>1}, 2).as(:v)).must_equal 1
+    ensure
+      @db.like_without_collate = like_without_collate
+    end
+  end
 end
 
 describe "A MSSQL database" do
@@ -28,7 +39,7 @@ describe "A MSSQL database" do
     @db = DB
     @db.create_table! :test3 do
       Integer :value
-      Time :time
+      TrueClass :b
     end
   end
   after do
@@ -38,6 +49,14 @@ describe "A MSSQL database" do
   it "should work with NOLOCK" do
     @db.transaction{@db[:test3].nolock.all.must_equal []}
   end
+
+  it "should emulate boolean columns" do
+    ds = @db[:test3]
+    ds.insert(:value=>1, :b=>true)
+    ds.insert(:value=>2, :b=>false)
+    ds.insert(:value=>3, :b=>nil)
+    ds.order(:value).select_map(:b).must_equal [true, false, nil]
+  end
 end
 
 describe "A MSSQL database" do
