@@ -51,6 +51,21 @@ describe "MySQL", '#create_table' do
     @db.schema(:dolls).map{|k, v| v[:auto_increment]}.must_equal [true, nil, nil]
   end
 
+  it "should support :on_update_current_timestamp column option" do
+    t = Time.now-60
+    @db.create_table(:dolls) do
+      Integer :id
+      timestamp :ts, :default=>t, :on_update_current_timestamp=>true
+      datetime :dt, :default=>t, :on_update_current_timestamp=>true
+    end
+    @db[:dolls].insert(:id=>1)
+    ts1, dt1 = @db[:dolls].get([:ts, :dt])
+    @db[:dolls].update(:id=>2)
+    ts2, dt2 = @db[:dolls].get([:ts, :dt])
+    ts1.wont_equal ts2
+    dt1.wont_equal dt2
+  end
+
   it "should support collate with various other column options" do
     @db.create_table!(:dolls){ String :name, :size=>128, :collate=>:utf8_bin, :default=>'foo', :null=>false, :unique=>true}
     @db[:dolls].insert
@@ -76,6 +91,11 @@ describe "MySQL", '#create_table' do
     @db.create_table(:dolls){String :a; String :b, generated_always_as: Sequel.function(:CONCAT, :a, 'plus')}
     @db.schema(:dolls).map{|_,v| v[:generated]}.must_equal [false, true]
   end
+
+  it "should include an :extra schema attribute" do
+    @db.create_table(:dolls) {Integer :a, :primary_key => true }
+    assert @db.schema(:dolls).first.last.key?(:extra)
+  end
 end
 
 if [:mysql, :mysql2].include?(DB.adapter_scheme)
@@ -523,10 +543,12 @@ describe "A MySQL database" do
     @db.create_table(:items) {String :a}
     @db.alter_table(:items){add_column :b, String, :generated_always_as=>Sequel.function(:CONCAT, :a, 'plus'), :generated_type=>:stored, :unique=>true}
     @db.schema(:items)[1][1][:generated].must_equal true
+    @db.schema(:items)[1][1][:extra].must_equal "STORED GENERATED"
     @db.alter_table(:items){add_column :c, String, :generated_always_as=>Sequel.function(:CONCAT, :a, 'minus'), :generated_type=>:virtual}
     @db.schema(:items)[2][1][:generated].must_equal true
-  end
-end  
+    @db.schema(:items)[2][1][:extra].must_equal "VIRTUAL GENERATED"
+  end if !DB.mariadb? || DB.server_version >= 100200
+end
 
 # Socket tests should only be run if the MySQL server is on localhost
 if DB.adapter_scheme == :mysql && %w'localhost 127.0.0.1 ::1'.include?(URI.parse(DB.uri).host)
@@ -1059,3 +1081,23 @@ describe "MySQL joined datasets" do
     @db[:b].select_order_map(:id).must_equal [5]
   end
 end
+
+describe "MySQL::Database#rename_tables" do
+  before do
+    @db = DB
+  end
+  after do
+    DB.drop_table?(:posts1, :messages1, :posts, :messages)
+  end
+
+  it "should rename multiple tables" do
+    @db.create_table!(:posts1){primary_key :a}
+    @db.create_table!(:messages1){primary_key :a}
+
+    @db.rename_tables([:posts1, :posts], [:messages1, :messages])
+    @db.table_exists?(:posts1).must_equal false
+    @db.table_exists?(:messages1).must_equal false
+    @db.table_exists?(:posts).must_equal true
+    @db.table_exists?(:messages).must_equal true
+  end
+end
