@@ -1,5 +1,4 @@
 require 'logger'
-require_relative "../sequel_warning"
 
 if ENV['COVERAGE']
   require_relative "../sequel_coverage"
@@ -23,6 +22,7 @@ Sequel::Database.extension :columns_introspection if ENV['SEQUEL_COLUMNS_INTROSP
 Sequel::Model.cache_associations = false if ENV['SEQUEL_NO_CACHE_ASSOCIATIONS']
 Sequel::Model.plugin :prepared_statements if ENV['SEQUEL_MODEL_PREPARED_STATEMENTS']
 Sequel::Model.plugin :throw_failures if ENV['SEQUEL_MODEL_THROW_FAILURES']
+Sequel::Model.plugin :primary_key_lookup_check_values if ENV['SEQUEL_PRIMARY_KEY_LOOKUP_CHECK_VALUES']
 Sequel::Model.cache_anonymous_models = false
 
 require_relative '../guards_helper'
