@@ -1525,3 +1525,20 @@ describe "Sequel core extensions" do
     Sequel.core_extensions?.must_equal false
   end
 end
+
+describe "Sequel.synchronize_with" do
+  it "should yield if given nil" do
+    Sequel.synchronize_with(nil){1}.must_equal 1
+  end
+
+  it "should synchronize with given mutex" do
+    m = Mutex.new
+    Sequel.synchronize_with(m){1}.must_equal 1
+
+    proc do
+      Sequel.synchronize_with(m) do
+        Sequel.synchronize_with(m){}
+      end
+    end.must_raise ThreadError
+  end
+end
