@@ -137,7 +137,7 @@ describe "Simple Dataset operations" do
   end
 
   it "should support iterating over large numbers of records with paged_each" do
-    v = (2..100).map{|i| wait{@ds.insert(:number=>i*10)}}
+    (2..100).map{|i| wait{@ds.insert(:number=>i*10)}}
 
     [:offset, :filter].each do |strategy|
       rows = []
@@ -855,6 +855,12 @@ if DB.dataset.supports_cte?
         @ds.except(@db[:t].with(:t, @ds)).select_order_map(:id).must_equal []
       end
     end
+
+    it "should give correct results for WITH AS [NOT] MATERIALIZED" do
+      @db[:t].with(:t, @ds, :materialized=>nil).order(:id).map(:id).must_equal [1, 2, 3, 4, 5, 6]
+      @db[:t].with(:t, @ds, :materialized=>true).order(:id).map(:id).must_equal [1, 2, 3, 4, 5, 6]
+      @db[:t].with(:t, @ds, :materialized=>false).order(:id).map(:id).must_equal [1, 2, 3, 4, 5, 6]
+    end if (DB.database_type == :postgres && DB.server_version >= 120000) || (DB.database_type == :sqlite && DB.sqlite_version > 33500)
   end
 end
 
@@ -1370,7 +1376,7 @@ describe "Sequel::Dataset main SQL methods" do
     @ds.select{[b, max(a).as(c)]}.group(:b).having{max(a) > 30}.all.each{|h| h[:c] = h[:c].to_i}.must_equal [{:b=>20, :c=>40}]
   end
   
-  cspecify "#having should work without a previous group", :sqlite do
+  cspecify "#having should work without a previous group", [proc{|db| db.sqlite_version < 33900}, :sqlite] do
     @ds = @ds.unordered
     @ds.select{max(a).as(c)}.having{max(a) > 30}.all.must_equal []
     wait{@ds.insert(20, 30)}
@@ -2131,3 +2137,117 @@ describe "Concurrent access" do
     end
   end
 end if [:threaded, :sharded_threaded].include?(DB.pool.pool_type) && DB.pool.max_size >= 4 && DB.database_type != :derby && DB.database_type != :sqlanywhere
+
+describe "MERGE" do
+  before(:all) do
+    @db = DB
+    @db.create_table!(:m1){Integer :i1; Integer :a}
+    @db.create_table!(:m2){Integer :i2; Integer :b}
+    @m1 = @db[:m1]
+    @m2 = @db[:m2]
+  end
+  after do
+    @m1.delete
+    @m2.delete
+  end
+  after(:all) do
+    @db.drop_table?(:m1, :m2)
+  end
+
+  def check(ds)
+    @m2.insert(1, 2)
+
+    @m1.all.must_equal []
+
+    # INSERT
+    ds.merge
+    @m1.all.must_equal [{:i1=>1, :a=>13}]
+
+    # UPDATE
+    ds.merge
+    @m1.all.must_equal [{:i1=>12, :a=>35}]
+
+    # DELETE MATCHING current row, INSERT NOT MATCHED new row
+    @m2.insert(12, 3)
+    ds.merge
+    @m1.all.must_equal [{:i1=>1, :a=>13}]
+
+    # MATCHED DO NOTHING
+    @m2.where(:i2=>12).delete
+    @m1.update(:a=>51)
+    ds.merge
+    @m1.all.must_equal [{:i1=>1, :a=>51}]
+
+    # NOT MATCHED DO NOTHING
+    @m1.delete
+    @m2.update(:b=>51)
+    ds.merge
+    @m1.all.must_equal []
+  end
+  
+  it "should allow inserts, updates, and deletes based on conditions in a single MERGE statement" do
+    ds = @m1.
+      merge_using(:m2, :i1=>:i2).
+      merge_insert(:i1=>Sequel[:i2], :a=>Sequel[:b]+11){b <= 50}.
+      merge_delete{{:a => 30..50}}.
+      merge_update(:i1=>Sequel[:i1]+:i2+10, :a=>Sequel[:a]+:b+20){a <= 50}
+    check(ds)
+  end
+
+  cspecify "should support WITH clauses", :db2 do
+    ds = @m1.
+      with(:m3, @db[:m2]).
+      merge_using(:m3, :i1=>:i2).
+      merge_insert(:i1=>Sequel[:i2], :a=>Sequel[:b]+11){b <= 50}.
+      merge_delete{{:a => 30..50}}.
+      merge_update(:i1=>Sequel[:i1]+:i2+10, :a=>Sequel[:a]+:b+20){a <= 50}
+    check(ds)
+  end if DB.dataset.supports_cte?
+
+  it "should support inserts with just columns" do
+    ds = @m1.
+      merge_using(:m2, :i1=>:i2).
+      merge_insert(Sequel[:i2], Sequel[:b]+11){b <= 50}.
+      merge_delete{{:a => 30..50}}.
+      merge_update(:i1=>Sequel[:i1]+:i2+10, :a=>Sequel[:a]+:b+20){a <= 50}
+    check(ds)
+  end
+
+  it "should calls inserts, updates, and deletes without conditions" do
+    @m2.insert(1, 2)
+    ds = @m1.merge_using(:m2, :i1=>:i2)
+    
+    ds.merge_insert(:i2, :b).merge
+    @m1.all.must_equal [{:i1=>1, :a=>2}]
+
+    ds.merge_update(:a=>Sequel[:a]+1).merge
+    @m1.all.must_equal [{:i1=>1, :a=>3}]
+
+    ds.merge_delete.merge
+    @m1.all.must_equal []
+  end
+
+  it "should raise if a merge is attempted without WHEN clauses" do
+    proc{@m1.merge_using(:m2, :i1=>:i2).merge}.must_raise Sequel::Error
+  end
+
+  it "should raise if a merge is attempted without a merge source" do
+    proc{@m1.merge_delete.merge}.must_raise Sequel::Error
+  end
+
+  it "should handle uncachable SQL" do
+    v = true
+    @m2.insert(1, 2)
+    ds = @m1.
+      merge_using(:m2, :i1=>:i2).
+      merge_insert(Sequel[:i2], Sequel[:b]+11){Sequel.delay{v}}
+
+    ds.merge
+    @m1.all.must_equal [{:i1=>1, :a=>13}]
+
+    @m1.delete
+    v = false
+    ds.merge
+    @m1.all.must_equal []
+  end
+end if DB.dataset.supports_merge? && DB.database_type != :oracle
