@@ -334,7 +334,12 @@ describe Sequel::Model do
 
   it "should raise an error on set_dataset if there is an error connecting to the database" do
     def @model.columns() raise Sequel::DatabaseConnectionError end
-    proc{@model.set_dataset(Sequel::Database.new[:foo].join(:blah).from_self)}.must_raise Sequel::DatabaseConnectionError
+    proc{@model.set_dataset(Sequel.mock[:foo].join(:blah).from_self)}.must_raise Sequel::DatabaseConnectionError
+  end
+
+  it "should raise an error on set_dataset if there is a disconnect error" do
+    def @model.columns() raise Sequel::DatabaseDisconnectError end
+    proc{@model.set_dataset(Sequel.mock[:foo].join(:blah).from_self)}.must_raise Sequel::DatabaseDisconnectError
   end
 
   it "should not raise an error if there is a problem getting the columns for a dataset" do
@@ -365,9 +370,9 @@ describe Sequel::Model do
     db = Sequel.mock
     def db.supports_savepoints?; true end
     Sequel::Model(db[:table])
-    db.sqls.must_equal ["SELECT * FROM table LIMIT 1"]
+    db.sqls.must_equal ["SELECT * FROM table LIMIT 0"]
     db.transaction{Sequel::Model(db[:table])}
-    db.sqls.must_equal ["BEGIN", "SAVEPOINT autopoint_1", "SELECT * FROM table LIMIT 1", "RELEASE SAVEPOINT autopoint_1", "COMMIT"]
+    db.sqls.must_equal ["BEGIN", "SAVEPOINT autopoint_1", "SELECT * FROM table LIMIT 0", "RELEASE SAVEPOINT autopoint_1", "COMMIT"]
   end
 
   it "should raise if bad inherited instance variable value is used" do
